from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...types import UNSET, Unset
from typing import Optional
from typing import Union
from ...models.export_study_method import ExportStudyMethod


def _get_kwargs(
    id: str,
    *,
    method: Union[Unset, None, ExportStudyMethod] = UNSET,
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    params: Dict[str, Any] = {}
    json_method: Union[Unset, None, str] = UNSET
    if not isinstance(method, Unset):
        json_method = method.value if method else None

    params["method"] = json_method

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/api/v1/studies/{id}/export/".format(
            id=id,
        ),
        "params": params,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[str]:
    if response.status_code == HTTPStatus.OK:
        response_200 = response.text
        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[str]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    method: Union[Unset, None, ExportStudyMethod] = UNSET,
    authorization: str,
) -> Response[str]:
    """Download demographic data

     __This is an evolving feature and the exact content of such exports is subject to change, so use at
    your own risk.__

    Download a snapshot of the participants' prescreening responses at the time that they took your
    study. Specify the optional `?method=EMAIL` query parameter if you want this to be sent to your
    email.

    In addition to the responses to all prescreeners applied to the study (subject to change), you'll
    also have access to the following data:

    * Submission id
    * Participant id
    * Submission status
    * Started date-time
      * Expressed in UTC
      * ISO 8601 formatted
    * Completed date-time
      * Expressed in UTC
      * ISO 8601 formatted
    * Time taken (in seconds)
    * Age
    * Sex
      * Participants were asked the following question: What is your sex, as recorded on legal/official
    documents?
    * First language
    * Current country of residence
    * Nationality
    * Country of birth
    * Student status
    * Employment status
    * Reviewed at date-time
      * Expressed in UTC
      * ISO 8601 formatted
    * Completion code ('entered code')

    Learn more on [Exporting Prolific demographic data](https://researcher-help.prolific.com/hc/en-
    gb/articles/360009391633-Exporting-Prolific-demographic-data).

    Args:
        id (str):
        method (Union[Unset, None, ExportStudyMethod]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[str]
    """

    kwargs = _get_kwargs(
        id=id,
        method=method,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: str,
    *,
    client: AuthenticatedClient,
    method: Union[Unset, None, ExportStudyMethod] = UNSET,
    authorization: str,
) -> Optional[str]:
    """Download demographic data

     __This is an evolving feature and the exact content of such exports is subject to change, so use at
    your own risk.__

    Download a snapshot of the participants' prescreening responses at the time that they took your
    study. Specify the optional `?method=EMAIL` query parameter if you want this to be sent to your
    email.

    In addition to the responses to all prescreeners applied to the study (subject to change), you'll
    also have access to the following data:

    * Submission id
    * Participant id
    * Submission status
    * Started date-time
      * Expressed in UTC
      * ISO 8601 formatted
    * Completed date-time
      * Expressed in UTC
      * ISO 8601 formatted
    * Time taken (in seconds)
    * Age
    * Sex
      * Participants were asked the following question: What is your sex, as recorded on legal/official
    documents?
    * First language
    * Current country of residence
    * Nationality
    * Country of birth
    * Student status
    * Employment status
    * Reviewed at date-time
      * Expressed in UTC
      * ISO 8601 formatted
    * Completion code ('entered code')

    Learn more on [Exporting Prolific demographic data](https://researcher-help.prolific.com/hc/en-
    gb/articles/360009391633-Exporting-Prolific-demographic-data).

    Args:
        id (str):
        method (Union[Unset, None, ExportStudyMethod]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        str
    """

    return sync_detailed(
        id=id,
        client=client,
        method=method,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    method: Union[Unset, None, ExportStudyMethod] = UNSET,
    authorization: str,
) -> Response[str]:
    """Download demographic data

     __This is an evolving feature and the exact content of such exports is subject to change, so use at
    your own risk.__

    Download a snapshot of the participants' prescreening responses at the time that they took your
    study. Specify the optional `?method=EMAIL` query parameter if you want this to be sent to your
    email.

    In addition to the responses to all prescreeners applied to the study (subject to change), you'll
    also have access to the following data:

    * Submission id
    * Participant id
    * Submission status
    * Started date-time
      * Expressed in UTC
      * ISO 8601 formatted
    * Completed date-time
      * Expressed in UTC
      * ISO 8601 formatted
    * Time taken (in seconds)
    * Age
    * Sex
      * Participants were asked the following question: What is your sex, as recorded on legal/official
    documents?
    * First language
    * Current country of residence
    * Nationality
    * Country of birth
    * Student status
    * Employment status
    * Reviewed at date-time
      * Expressed in UTC
      * ISO 8601 formatted
    * Completion code ('entered code')

    Learn more on [Exporting Prolific demographic data](https://researcher-help.prolific.com/hc/en-
    gb/articles/360009391633-Exporting-Prolific-demographic-data).

    Args:
        id (str):
        method (Union[Unset, None, ExportStudyMethod]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[str]
    """

    kwargs = _get_kwargs(
        id=id,
        method=method,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: str,
    *,
    client: AuthenticatedClient,
    method: Union[Unset, None, ExportStudyMethod] = UNSET,
    authorization: str,
) -> Optional[str]:
    """Download demographic data

     __This is an evolving feature and the exact content of such exports is subject to change, so use at
    your own risk.__

    Download a snapshot of the participants' prescreening responses at the time that they took your
    study. Specify the optional `?method=EMAIL` query parameter if you want this to be sent to your
    email.

    In addition to the responses to all prescreeners applied to the study (subject to change), you'll
    also have access to the following data:

    * Submission id
    * Participant id
    * Submission status
    * Started date-time
      * Expressed in UTC
      * ISO 8601 formatted
    * Completed date-time
      * Expressed in UTC
      * ISO 8601 formatted
    * Time taken (in seconds)
    * Age
    * Sex
      * Participants were asked the following question: What is your sex, as recorded on legal/official
    documents?
    * First language
    * Current country of residence
    * Nationality
    * Country of birth
    * Student status
    * Employment status
    * Reviewed at date-time
      * Expressed in UTC
      * ISO 8601 formatted
    * Completion code ('entered code')

    Learn more on [Exporting Prolific demographic data](https://researcher-help.prolific.com/hc/en-
    gb/articles/360009391633-Exporting-Prolific-demographic-data).

    Args:
        id (str):
        method (Union[Unset, None, ExportStudyMethod]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        str
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            method=method,
            authorization=authorization,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field


from ..models.request_return_action import RequestReturnAction


T = TypeVar("T", bound="RequestReturn")


@_attrs_define
class RequestReturn:
    """
    Attributes:
        action (RequestReturnAction): The action to take
        return_reason (str): The reason you would like to request a return Example: Failed an attention check.
    """

    action: RequestReturnAction
    return_reason: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        action = self.action.value

        return_reason = self.return_reason

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "action": action,
                "return_reason": return_reason,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        action = RequestReturnAction(d.pop("action"))

        return_reason = d.pop("return_reason")

        request_return = cls(
            action=action,
            return_reason=return_reason,
        )

        request_return.additional_properties = d
        return request_return

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.submission_status import SubmissionStatus
from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="Submission")


@_attrs_define
class Submission:
    """
    Example:
        {'id': '625d4a831bcda2d59ac5a251', 'completed_at': datetime.datetime(2022, 4, 18, 11, 25, 2, 734000,
            tzinfo=datetime.timezone.utc), 'entered_code': '8E8AC860', 'participant': '60bf9310e8dec401be6e9615',
            'started_at': datetime.datetime(2022, 4, 18, 11, 24, 51, 395000, tzinfo=datetime.timezone.utc), 'status':
            'APPROVED', 'study_id': '60aca280709ee40ec37d4885'}

    Attributes:
        id (str): The id of the submission
        started_at (str): The date and time that the user started the submission (UTC)
        status (SubmissionStatus): The current status of the submission
        study_id (str): Study id.
        completed_at (Union[Unset, None, str]): The time the submission was completed at.
        entered_code (Union[Unset, None, str]): The completion code used by the participant to complete the study.
        participant (Union[Unset, str]): Participant id.
    """

    id: str
    started_at: str
    status: SubmissionStatus
    study_id: str
    completed_at: Union[Unset, None, str] = UNSET
    entered_code: Union[Unset, None, str] = UNSET
    participant: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        started_at = self.started_at
        status = self.status.value

        study_id = self.study_id
        completed_at = self.completed_at
        entered_code = self.entered_code
        participant = self.participant

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "started_at": started_at,
                "status": status,
                "study_id": study_id,
            }
        )
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if entered_code is not UNSET:
            field_dict["entered_code"] = entered_code
        if participant is not UNSET:
            field_dict["participant"] = participant

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        started_at = d.pop("started_at")

        status = SubmissionStatus(d.pop("status"))

        study_id = d.pop("study_id")

        completed_at = d.pop("completed_at", UNSET)

        entered_code = d.pop("entered_code", UNSET)

        participant = d.pop("participant", UNSET)

        submission = cls(
            id=id,
            started_at=started_at,
            status=status,
            study_id=study_id,
            completed_at=completed_at,
            entered_code=entered_code,
            participant=participant,
        )

        submission.additional_properties = d
        return submission

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from typing import List
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.participant_group_feeder_studies_item import (
        ParticipantGroupFeederStudiesItem,
    )


T = TypeVar("T", bound="ParticipantGroup")


@_attrs_define
class ParticipantGroup:
    """
    Attributes:
        id (Union[Unset, str]): The id of the participant group Example: 5e9b9c9b0f9c9a0001b0b1f5.
        name (Union[Unset, str]): The name of the participant group Example: Group 1.
        project_id (Union[Unset, None, str]): The id of the project the participant group belongs to
        workspace_id (Union[Unset, None, str]): The id of the workspace the participant group belongs to. A participant
            group can only belong to either a workspace or an organisation. Example: 5e9b9c9b0f9c9a0001b1ca2f.
        organisation_id (Union[Unset, None, str]): The id of the organisation the participant group belongs to. A
            participant group can only belong to either a workspace or an organisation. Example: 5e9b9c9b0f9c9a0001b1ca2f.
        description (Union[Unset, None, str]): The user-provided description of the participant group Example: My first
            participant group.
        participant_count (Union[Unset, int]): The number of participants in the participant group Example: 10.
        is_deleted (Union[Unset, bool]): Whether the participant group has been deleted
        feeder_studies (Union[Unset, List['ParticipantGroupFeederStudiesItem']]): Details of all studies which are
            configured to modify the participants in this group through completion codes.
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    project_id: Union[Unset, None, str] = UNSET
    workspace_id: Union[Unset, None, str] = UNSET
    organisation_id: Union[Unset, None, str] = UNSET
    description: Union[Unset, None, str] = UNSET
    participant_count: Union[Unset, int] = UNSET
    is_deleted: Union[Unset, bool] = UNSET
    feeder_studies: Union[Unset, List["ParticipantGroupFeederStudiesItem"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        project_id = self.project_id
        workspace_id = self.workspace_id
        organisation_id = self.organisation_id
        description = self.description
        participant_count = self.participant_count
        is_deleted = self.is_deleted
        feeder_studies: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.feeder_studies, Unset):
            feeder_studies = []
            for feeder_studies_item_data in self.feeder_studies:
                feeder_studies_item = feeder_studies_item_data.to_dict()

                feeder_studies.append(feeder_studies_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if project_id is not UNSET:
            field_dict["project_id"] = project_id
        if workspace_id is not UNSET:
            field_dict["workspace_id"] = workspace_id
        if organisation_id is not UNSET:
            field_dict["organisation_id"] = organisation_id
        if description is not UNSET:
            field_dict["description"] = description
        if participant_count is not UNSET:
            field_dict["participant_count"] = participant_count
        if is_deleted is not UNSET:
            field_dict["is_deleted"] = is_deleted
        if feeder_studies is not UNSET:
            field_dict["feeder_studies"] = feeder_studies

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.participant_group_feeder_studies_item import (
            ParticipantGroupFeederStudiesItem,
        )

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        project_id = d.pop("project_id", UNSET)

        workspace_id = d.pop("workspace_id", UNSET)

        organisation_id = d.pop("organisation_id", UNSET)

        description = d.pop("description", UNSET)

        participant_count = d.pop("participant_count", UNSET)

        is_deleted = d.pop("is_deleted", UNSET)

        feeder_studies = []
        _feeder_studies = d.pop("feeder_studies", UNSET)
        for feeder_studies_item_data in _feeder_studies or []:
            feeder_studies_item = ParticipantGroupFeederStudiesItem.from_dict(
                feeder_studies_item_data
            )

            feeder_studies.append(feeder_studies_item)

        participant_group = cls(
            id=id,
            name=name,
            project_id=project_id,
            workspace_id=workspace_id,
            organisation_id=organisation_id,
            description=description,
            participant_count=participant_count,
            is_deleted=is_deleted,
            feeder_studies=feeder_studies,
        )

        participant_group.additional_properties = d
        return participant_group

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SubmissionStatus(str, Enum):
    ACTIVE = "ACTIVE"
    APPROVED = "APPROVED"
    AWAITING_REVIEW = "AWAITING REVIEW"
    REJECTED = "REJECTED"
    RESERVED = "RESERVED"
    RETURNED = "RETURNED"
    TIMED_OUT = "TIMED-OUT"
    UNKNOWN = "UNKNOWN"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.question_type import QuestionType
from typing import List
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.answer_option import AnswerOption


T = TypeVar("T", bound="Question")


@_attrs_define
class Question:
    """Responsible for defining a question within a survey.

    Attributes:
        answers (List['AnswerOption']): An array of answer options for a question.
        title (str): The question title. Example: What is your favourite root vegetable?.
        type (QuestionType): Responsible for articulating the question type. At the moment we have:

            - single answer
            - multiple answer

            Args:
                str (_type_): The type of question.
                Enum (_type_): The class to define an enum.
        id (Union[Unset, str]):
    """

    answers: List["AnswerOption"]
    title: str
    type: QuestionType
    id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        answers = []
        for answers_item_data in self.answers:
            answers_item = answers_item_data.to_dict()

            answers.append(answers_item)

        title = self.title
        type = self.type.value

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "answers": answers,
                "title": title,
                "type": type,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.answer_option import AnswerOption

        d = src_dict.copy()
        answers = []
        _answers = d.pop("answers")
        for answers_item_data in _answers:
            answers_item = AnswerOption.from_dict(answers_item_data)

            answers.append(answers_item)

        title = d.pop("title")

        type = QuestionType(d.pop("type"))

        id = d.pop("id", UNSET)

        question = cls(
            answers=answers,
            title=title,
            type=type,
            id=id,
        )

        question.additional_properties = d
        return question

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response
from ... import errors

from typing import Dict
from ...models.duplicate_study_json_body import DuplicateStudyJsonBody
from ...models.study import Study


def _get_kwargs(
    id: str,
    *,
    json_body: DuplicateStudyJsonBody,
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": "/api/v1/studies/{id}/clone/".format(
            id=id,
        ),
        "json": json_json_body,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Study]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Study.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Study]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: DuplicateStudyJsonBody,
    authorization: str,
) -> Response[Study]:
    """Duplicate a study

     Duplicating a study can save a lot of time and effort compared to creating a new study from scratch,
    since all of the information and data from the original study will be carried over to the new study.
    Additionally, duplicating a study can help ensure that the new study is set up consistently with the
    original, reducing the risk of errors or inconsistencies.

    To prevent repeat participation, a block list containing the original study will be added to the
    duplicated study's filters unless `block_previous_participants` is set to `false` in the request
    body.

    Args:
        id (str):
        authorization (str):
        json_body (DuplicateStudyJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Study]
    """

    kwargs = _get_kwargs(
        id=id,
        json_body=json_body,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: DuplicateStudyJsonBody,
    authorization: str,
) -> Optional[Study]:
    """Duplicate a study

     Duplicating a study can save a lot of time and effort compared to creating a new study from scratch,
    since all of the information and data from the original study will be carried over to the new study.
    Additionally, duplicating a study can help ensure that the new study is set up consistently with the
    original, reducing the risk of errors or inconsistencies.

    To prevent repeat participation, a block list containing the original study will be added to the
    duplicated study's filters unless `block_previous_participants` is set to `false` in the request
    body.

    Args:
        id (str):
        authorization (str):
        json_body (DuplicateStudyJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Study
    """

    return sync_detailed(
        id=id,
        client=client,
        json_body=json_body,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: DuplicateStudyJsonBody,
    authorization: str,
) -> Response[Study]:
    """Duplicate a study

     Duplicating a study can save a lot of time and effort compared to creating a new study from scratch,
    since all of the information and data from the original study will be carried over to the new study.
    Additionally, duplicating a study can help ensure that the new study is set up consistently with the
    original, reducing the risk of errors or inconsistencies.

    To prevent repeat participation, a block list containing the original study will be added to the
    duplicated study's filters unless `block_previous_participants` is set to `false` in the request
    body.

    Args:
        id (str):
        authorization (str):
        json_body (DuplicateStudyJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Study]
    """

    kwargs = _get_kwargs(
        id=id,
        json_body=json_body,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: DuplicateStudyJsonBody,
    authorization: str,
) -> Optional[Study]:
    """Duplicate a study

     Duplicating a study can save a lot of time and effort compared to creating a new study from scratch,
    since all of the information and data from the original study will be carried over to the new study.
    Additionally, duplicating a study can help ensure that the new study is set up consistently with the
    original, reducing the risk of errors or inconsistencies.

    To prevent repeat participation, a block list containing the original study will be added to the
    duplicated study's filters unless `block_previous_participants` is set to `false` in the request
    body.

    Args:
        id (str):
        authorization (str):
        json_body (DuplicateStudyJsonBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Study
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            json_body=json_body,
            authorization=authorization,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.message_data_category import MessageDataCategory
from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="MessageData")


@_attrs_define
class MessageData:
    """Metadata for a message

    Attributes:
        study_id (Union[Unset, str]): What study the message relates to. In case this is not automatically filled for
            the participant, they can choose which study their message relates to. Example: 620ca2735fcbba4fa2b3211a.
        category (Union[Unset, MessageDataCategory]): Participants can self-categorise their message before sending it.
            Example: feedback.
    """

    study_id: Union[Unset, str] = UNSET
    category: Union[Unset, MessageDataCategory] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        study_id = self.study_id
        category: Union[Unset, str] = UNSET
        if not isinstance(self.category, Unset):
            category = self.category.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if study_id is not UNSET:
            field_dict["study_id"] = study_id
        if category is not UNSET:
            field_dict["category"] = category

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        study_id = d.pop("study_id", UNSET)

        _category = d.pop("category", UNSET)
        category: Union[Unset, MessageDataCategory]
        if isinstance(_category, Unset):
            category = UNSET
        else:
            category = MessageDataCategory(_category)

        message_data = cls(
            study_id=study_id,
            category=category,
        )

        message_data.additional_properties = d
        return message_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from typing import List
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.summary_question import SummaryQuestion


T = TypeVar("T", bound="Summary")


@_attrs_define
class Summary:
    """Responsible for providing a base for all the aggregated answers for a survey.

    Example:
        {'survey_id': '63346b3a4fd1fe7b39f192e9', 'questions': [{'question_id': '02dee012-25e4-449e-8f2f-a552b9007d92',
            'question': 'Do you feel that this will all be alright on the night?', 'total_answers': 10, 'answers':
            [{'answer_id': '4c69011b-9b6b-46c6-8ba2-da80761131dc', 'answer': True, 'count': 4}, {'answer_id':
            'f3c8d38f-842c-49d9-a0bb-e17b7803b240', 'answer': False, 'count': 6}]}]}

    Attributes:
        survey_id (str): The survey ID.
        questions (Union[Unset, List['SummaryQuestion']]): A list of questions for the given survey.
    """

    survey_id: str
    questions: Union[Unset, List["SummaryQuestion"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        survey_id = self.survey_id
        questions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.questions, Unset):
            questions = []
            for questions_item_data in self.questions:
                questions_item = questions_item_data.to_dict()

                questions.append(questions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "survey_id": survey_id,
            }
        )
        if questions is not UNSET:
            field_dict["questions"] = questions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.summary_question import SummaryQuestion

        d = src_dict.copy()
        survey_id = d.pop("survey_id")

        questions = []
        _questions = d.pop("questions", UNSET)
        for questions_item_data in _questions or []:
            questions_item = SummaryQuestion.from_dict(questions_item_data)

            questions.append(questions_item)

        summary = cls(
            survey_id=survey_id,
            questions=questions,
        )

        summary.additional_properties = d
        return summary

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field


from typing import List
from typing import Dict

if TYPE_CHECKING:
    from ..models.question_response import QuestionResponse


T = TypeVar("T", bound="Section")


@_attrs_define
class Section:
    """Responsible for linking question/answers to a response for a survey.

    This is more of a long term thing, but helps if we add now.

        Attributes:
            questions (List['QuestionResponse']): The questions for a given section.
            section_id (str): The section ID.
    """

    questions: List["QuestionResponse"]
    section_id: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        questions = []
        for questions_item_data in self.questions:
            questions_item = questions_item_data.to_dict()

            questions.append(questions_item)

        section_id = self.section_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "questions": questions,
                "section_id": section_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.question_response import QuestionResponse

        d = src_dict.copy()
        questions = []
        _questions = d.pop("questions")
        for questions_item_data in _questions:
            questions_item = QuestionResponse.from_dict(questions_item_data)

            questions.append(questions_item)

        section_id = d.pop("section_id")

        section = cls(
            questions=questions,
            section_id=section_id,
        )

        section.additional_properties = d
        return section

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...types import UNSET, Unset
from ...models.messages import Messages
from typing import Union
from typing import Optional
from typing import Dict


def _get_kwargs(
    *,
    user_id: Union[Unset, None, str] = UNSET,
    created_after: Union[Unset, None, str] = UNSET,
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    params: Dict[str, Any] = {}
    params["user_id"] = user_id

    params["created_after"] = created_after

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/api/v1/messages/",
        "params": params,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Messages]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Messages.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Messages]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    user_id: Union[Unset, None, str] = UNSET,
    created_after: Union[Unset, None, str] = UNSET,
    authorization: str,
) -> Response[Messages]:
    """Retrieve messages

     Get messages between you and another user or your messages with all users.

    Args:
        user_id (Union[Unset, None, str]):
        created_after (Union[Unset, None, str]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Messages]
    """

    kwargs = _get_kwargs(
        user_id=user_id,
        created_after=created_after,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    user_id: Union[Unset, None, str] = UNSET,
    created_after: Union[Unset, None, str] = UNSET,
    authorization: str,
) -> Optional[Messages]:
    """Retrieve messages

     Get messages between you and another user or your messages with all users.

    Args:
        user_id (Union[Unset, None, str]):
        created_after (Union[Unset, None, str]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Messages
    """

    return sync_detailed(
        client=client,
        user_id=user_id,
        created_after=created_after,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    user_id: Union[Unset, None, str] = UNSET,
    created_after: Union[Unset, None, str] = UNSET,
    authorization: str,
) -> Response[Messages]:
    """Retrieve messages

     Get messages between you and another user or your messages with all users.

    Args:
        user_id (Union[Unset, None, str]):
        created_after (Union[Unset, None, str]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Messages]
    """

    kwargs = _get_kwargs(
        user_id=user_id,
        created_after=created_after,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    user_id: Union[Unset, None, str] = UNSET,
    created_after: Union[Unset, None, str] = UNSET,
    authorization: str,
) -> Optional[Messages]:
    """Retrieve messages

     Get messages between you and another user or your messages with all users.

    Args:
        user_id (Union[Unset, None, str]):
        created_after (Union[Unset, None, str]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Messages
    """

    return (
        await asyncio_detailed(
            client=client,
            user_id=user_id,
            created_after=created_after,
            authorization=authorization,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="BaseStudySubmissionsConfig")


@_attrs_define
class BaseStudySubmissionsConfig:
    """**BETA**: This is a beta feature and is currently only available to selected workspaces.
    It is being tested and evaluated for effectiveness and user experience before being released to all users.

    **Advanced**: This helps with faster data collection. Your survey system will need to handle providing a
    unique experience each time the participant takes the study.

    Configuration related to study submissions. The purpose of this field is to capture any configuration options that
    impact the submissions made by participants in a study.

        Attributes:
            max_submissions_per_participant (Union[Unset, None, int]): - **1** is the default Prolific experience. This
                means one submission, per participant, per study. If you do
                  not specify this field, the **default is 1**.
                - **1+** turns your study into a multi-submission study, meaning a participant can create many submissions per
                study.
                  As noted above, your survey system will need to handle providing a
                unique experience each time the participant takes the study.
                - **-1** will allow an indefinite number of submissions from a single participant, up to
                `total_available_places`. Default: 1.
            max_concurrent_submissions (Union[Unset, None, int]): - **-1** is the default value, meaning unlimited
                concurrent active/reserved submissions per study.
                - **1+** limits the number of concurrent active/reserved submissions a study can have at one time. Default: -1.
    """

    max_submissions_per_participant: Union[Unset, None, int] = 1
    max_concurrent_submissions: Union[Unset, None, int] = -1
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        max_submissions_per_participant = self.max_submissions_per_participant
        max_concurrent_submissions = self.max_concurrent_submissions

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if max_submissions_per_participant is not UNSET:
            field_dict[
                "max_submissions_per_participant"
            ] = max_submissions_per_participant
        if max_concurrent_submissions is not UNSET:
            field_dict["max_concurrent_submissions"] = max_concurrent_submissions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        max_submissions_per_participant = d.pop(
            "max_submissions_per_participant", UNSET
        )

        max_concurrent_submissions = d.pop("max_concurrent_submissions", UNSET)

        base_study_submissions_config = cls(
            max_submissions_per_participant=max_submissions_per_participant,
            max_concurrent_submissions=max_concurrent_submissions,
        )

        base_study_submissions_config.additional_properties = d
        return base_study_submissions_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from typing import List
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.section import Section
    from ..models.question_response import QuestionResponse


T = TypeVar("T", bound="ResponseIn")


@_attrs_define
class ResponseIn:
    """The model used to create a `Response`.

    Attributes:
        participant_id (str): The Prolific participant ID. Example: 62908f0b98a55b36ac68b992.
        submission_id (str): The Prolific submission ID. Example: 62908f0b98a55b36ac68b992.
        sections (Union[Unset, List['Section']]): An array of sections from the survey, otherwise use `questions`.
        questions (Union[Unset, List['QuestionResponse']]): An array of questions from the survey, otherwise use
            `sections`.
    """

    participant_id: str
    submission_id: str
    sections: Union[Unset, List["Section"]] = UNSET
    questions: Union[Unset, List["QuestionResponse"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        participant_id = self.participant_id
        submission_id = self.submission_id
        sections: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.sections, Unset):
            sections = []
            for sections_item_data in self.sections:
                sections_item = sections_item_data.to_dict()

                sections.append(sections_item)

        questions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.questions, Unset):
            questions = []
            for questions_item_data in self.questions:
                questions_item = questions_item_data.to_dict()

                questions.append(questions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "participant_id": participant_id,
                "submission_id": submission_id,
            }
        )
        if sections is not UNSET:
            field_dict["sections"] = sections
        if questions is not UNSET:
            field_dict["questions"] = questions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.section import Section
        from ..models.question_response import QuestionResponse

        d = src_dict.copy()
        participant_id = d.pop("participant_id")

        submission_id = d.pop("submission_id")

        sections = []
        _sections = d.pop("sections", UNSET)
        for sections_item_data in _sections or []:
            sections_item = Section.from_dict(sections_item_data)

            sections.append(sections_item)

        questions = []
        _questions = d.pop("questions", UNSET)
        for questions_item_data in _questions or []:
            questions_item = QuestionResponse.from_dict(questions_item_data)

            questions.append(questions_item)

        response_in = cls(
            participant_id=participant_id,
            submission_id=submission_id,
            sections=sections,
            questions=questions,
        )

        response_in.additional_properties = d
        return response_in

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.participant_group_feeder_studies_item_feeder_completion_codes_item_action import (
    ParticipantGroupFeederStudiesItemFeederCompletionCodesItemAction,
)
from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="ParticipantGroupFeederStudiesItemFeederCompletionCodesItem")


@_attrs_define
class ParticipantGroupFeederStudiesItemFeederCompletionCodesItem:
    """The code within this study that interacts with the participant group.

    Attributes:
        code (Union[Unset, str]): The code that will modify the participants in this group. Example: AJVRH234.
        code_type (Union[Unset, str]): The label or code type given to this code within the context of the study.
            Example: COMPLETION_CODE.
        action (Union[Unset, ParticipantGroupFeederStudiesItemFeederCompletionCodesItemAction]): The action that will be
            taken when this code is used. Example: ADD_TO_PARTICIPANT_GROUP.
    """

    code: Union[Unset, str] = UNSET
    code_type: Union[Unset, str] = UNSET
    action: Union[
        Unset, ParticipantGroupFeederStudiesItemFeederCompletionCodesItemAction
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        code = self.code
        code_type = self.code_type
        action: Union[Unset, str] = UNSET
        if not isinstance(self.action, Unset):
            action = self.action.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if code is not UNSET:
            field_dict["code"] = code
        if code_type is not UNSET:
            field_dict["code_type"] = code_type
        if action is not UNSET:
            field_dict["action"] = action

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        code = d.pop("code", UNSET)

        code_type = d.pop("code_type", UNSET)

        _action = d.pop("action", UNSET)
        action: Union[
            Unset, ParticipantGroupFeederStudiesItemFeederCompletionCodesItemAction
        ]
        if isinstance(_action, Unset):
            action = UNSET
        else:
            action = ParticipantGroupFeederStudiesItemFeederCompletionCodesItemAction(
                _action
            )

        participant_group_feeder_studies_item_feeder_completion_codes_item = cls(
            code=code,
            code_type=code_type,
            action=action,
        )

        participant_group_feeder_studies_item_feeder_completion_codes_item.additional_properties = d
        return participant_group_feeder_studies_item_feeder_completion_codes_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...types import UNSET, Unset
from typing import Union
from ...models.send_bulk_message import SendBulkMessage
from typing import Dict


def _get_kwargs(
    *,
    json_body: SendBulkMessage,
    authorization: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    headers = {}
    if not isinstance(authorization, Unset):
        headers["Authorization"] = authorization

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": "/api/v1/messages/bulk/",
        "json": json_json_body,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Any]:
    if response.status_code == HTTPStatus.NO_CONTENT:
        return None
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Any]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    json_body: SendBulkMessage,
    authorization: Union[Unset, str] = UNSET,
) -> Response[Any]:
    """Send a message to multiple participants

     Send a message to multiple participants.

    Args:
        authorization (Union[Unset, str]):
        json_body (SendBulkMessage):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    json_body: SendBulkMessage,
    authorization: Union[Unset, str] = UNSET,
) -> Response[Any]:
    """Send a message to multiple participants

     Send a message to multiple participants.

    Args:
        authorization (Union[Unset, str]):
        json_body (SendBulkMessage):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)



from enum import Enum


class MessageDataCategory(str, Enum):
    FEEDBACK = "feedback"
    OTHER = "other"
    PAYMENT_ISSUES = "payment-issues"
    PAYMENT_TIMING = "payment-timing"
    REJECTIONS = "rejections"
    TECHNICAL_ISSUES = "technical-issues"

    def __str__(self) -> str:
        return str(self.value)



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...types import UNSET, Unset
from typing import Union
from ...models.get_filter_set_response_200 import GetFilterSetResponse200
from typing import Optional
from typing import Dict


def _get_kwargs(
    id: str,
    *,
    version_number: Union[Unset, None, int] = UNSET,
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    params: Dict[str, Any] = {}
    params["version_number"] = version_number

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": "/api/v1/filter-sets/{id}/".format(
            id=id,
        ),
        "params": params,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[GetFilterSetResponse200]:
    if response.status_code == HTTPStatus.OK:
        response_200 = GetFilterSetResponse200.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[GetFilterSetResponse200]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    version_number: Union[Unset, None, int] = UNSET,
    authorization: str,
) -> Response[GetFilterSetResponse200]:
    """Get filter set

     Get details of a filter set.

    Args:
        id (str):
        version_number (Union[Unset, None, int]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[GetFilterSetResponse200]
    """

    kwargs = _get_kwargs(
        id=id,
        version_number=version_number,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: str,
    *,
    client: AuthenticatedClient,
    version_number: Union[Unset, None, int] = UNSET,
    authorization: str,
) -> Optional[GetFilterSetResponse200]:
    """Get filter set

     Get details of a filter set.

    Args:
        id (str):
        version_number (Union[Unset, None, int]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        GetFilterSetResponse200
    """

    return sync_detailed(
        id=id,
        client=client,
        version_number=version_number,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    version_number: Union[Unset, None, int] = UNSET,
    authorization: str,
) -> Response[GetFilterSetResponse200]:
    """Get filter set

     Get details of a filter set.

    Args:
        id (str):
        version_number (Union[Unset, None, int]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[GetFilterSetResponse200]
    """

    kwargs = _get_kwargs(
        id=id,
        version_number=version_number,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: str,
    *,
    client: AuthenticatedClient,
    version_number: Union[Unset, None, int] = UNSET,
    authorization: str,
) -> Optional[GetFilterSetResponse200]:
    """Get filter set

     Get details of a filter set.

    Args:
        id (str):
        version_number (Union[Unset, None, int]):
        authorization (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        GetFilterSetResponse200
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            version_number=version_number,
            authorization=authorization,
        )
    ).parsed



from enum import Enum


class SubmissionTransitionRejectionCategory(str, Enum):
    BAD_CODE = "BAD_CODE"
    FAILED_CHECK = "FAILED_CHECK"
    FAILED_INSTRUCTIONS = "FAILED_INSTRUCTIONS"
    INCOMP_LONGITUDINAL = "INCOMP_LONGITUDINAL"
    LOW_EFFORT = "LOW_EFFORT"
    MALINGERING = "MALINGERING"
    NO_CODE = "NO_CODE"
    NO_DATA = "NO_DATA"
    OTHER = "OTHER"
    TOO_QUICKLY = "TOO_QUICKLY"
    TOO_SLOWLY = "TOO_SLOWLY"
    UNSUPP_DEVICE = "UNSUPP_DEVICE"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
import datetime
from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import Dict

if TYPE_CHECKING:
    from ..models.message_data import MessageData


T = TypeVar("T", bound="Message")


@_attrs_define
class Message:
    """
    Attributes:
        sender_id (str): Id of the user who sent the message
        body (str): Body of the message.
        sent_at (datetime.datetime): Date time when message was sent
        channel_id (str): The channel ID, for linking back to a thread in the Prolific app. Example:
            d45c8a5e812ff990fc6546beaf888c9820f4c184f7200a45d900cf0f321f7f38.
        type (Union[Unset, str]): Will only me message for now
        data (Union[Unset, MessageData]): Metadata for a message
    """

    sender_id: str
    body: str
    sent_at: datetime.datetime
    channel_id: str
    type: Union[Unset, str] = UNSET
    data: Union[Unset, "MessageData"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        sender_id = self.sender_id
        body = self.body
        sent_at = self.sent_at.isoformat()

        channel_id = self.channel_id
        type = self.type
        data: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data, Unset):
            data = self.data.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "sender_id": sender_id,
                "body": body,
                "sent_at": sent_at,
                "channel_id": channel_id,
            }
        )
        if type is not UNSET:
            field_dict["type"] = type
        if data is not UNSET:
            field_dict["data"] = data

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.message_data import MessageData

        d = src_dict.copy()
        sender_id = d.pop("sender_id")

        body = d.pop("body")

        sent_at = isoparse(d.pop("sent_at"))

        channel_id = d.pop("channel_id")

        type = d.pop("type", UNSET)

        _data = d.pop("data", UNSET)
        data: Union[Unset, MessageData]
        if isinstance(_data, Unset):
            data = UNSET
        else:
            data = MessageData.from_dict(_data)

        message = cls(
            sender_id=sender_id,
            body=body,
            sent_at=sent_at,
            channel_id=channel_id,
            type=type,
            data=data,
        )

        message.additional_properties = d
        return message

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="ParticipantGroupMembership")


@_attrs_define
class ParticipantGroupMembership:
    """
    Attributes:
        participant_id (Union[Unset, str]): The id of the participant Example: 5e9b9c9b0f9c9a0001b0b1f5.
        datetime_created (Union[Unset, str]): The date and time the participant was added to the Participant Group
            Example: 2020-04-20T12:00:00Z.
    """

    participant_id: Union[Unset, str] = UNSET
    datetime_created: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        participant_id = self.participant_id
        datetime_created = self.datetime_created

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if participant_id is not UNSET:
            field_dict["participant_id"] = participant_id
        if datetime_created is not UNSET:
            field_dict["datetime_created"] = datetime_created

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        participant_id = d.pop("participant_id", UNSET)

        datetime_created = d.pop("datetime_created", UNSET)

        participant_group_membership = cls(
            participant_id=participant_id,
            datetime_created=datetime_created,
        )

        participant_group_membership.additional_properties = d
        return participant_group_membership

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class BaseStudyPeripheralRequirementsItem(str, Enum):
    AUDIO = "audio"
    CAMERA = "camera"
    DOWNLOAD = "download"
    MICROPHONE = "microphone"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast, List
from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="CreateParticipantGroupJsonBody")


@_attrs_define
class CreateParticipantGroupJsonBody:
    """
    Attributes:
        name (str): The name of the participant group Example: Group 1.
        workspace_id (Union[Unset, str]): The id of the workspace to create the participant group in. Either a workspace
            or organisation ID must be specified.
        organisation_id (Union[Unset, str]): The id of the organisation to create the participant group in. Either a
            workspace or organisation ID must be specified.
        description (Union[Unset, str]): A description of the participant group Example: Participants with confirmed
            special dietary requirements..
        participant_ids (Union[Unset, List[str]]): The ids of participants to be initially added to the group Example:
            ['5e9b9c9b0f9c9a0001b0b1f4', '5e9b9c9b0f9c9a0001b0b1f5', '5e9b9c9b0f9c9a0001b0b1f6'].
    """

    name: str
    workspace_id: Union[Unset, str] = UNSET
    organisation_id: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    participant_ids: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        workspace_id = self.workspace_id
        organisation_id = self.organisation_id
        description = self.description
        participant_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.participant_ids, Unset):
            participant_ids = self.participant_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if workspace_id is not UNSET:
            field_dict["workspace_id"] = workspace_id
        if organisation_id is not UNSET:
            field_dict["organisation_id"] = organisation_id
        if description is not UNSET:
            field_dict["description"] = description
        if participant_ids is not UNSET:
            field_dict["participant_ids"] = participant_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        workspace_id = d.pop("workspace_id", UNSET)

        organisation_id = d.pop("organisation_id", UNSET)

        description = d.pop("description", UNSET)

        participant_ids = cast(List[str], d.pop("participant_ids", UNSET))

        create_participant_group_json_body = cls(
            name=name,
            workspace_id=workspace_id,
            organisation_id=organisation_id,
            description=description,
            participant_ids=participant_ids,
        )

        create_participant_group_json_body.additional_properties = d
        return create_participant_group_json_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
import datetime
from typing import List
from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import Dict

if TYPE_CHECKING:
    from ..models.section import Section
    from ..models.question_response import QuestionResponse


T = TypeVar("T", bound="ResponseOut")


@_attrs_define
class ResponseOut:
    """The model used to create a serialised representation a `Response`.

    Attributes:
        participant_id (str): The Prolific participant ID. Example: 62908f0b98a55b36ac68b992.
        submission_id (str): The Prolific submission ID. Example: 62908f0b98a55b36ac68b992.
        field_id (Union[Unset, str]):
        date_created (Union[Unset, datetime.datetime]): The date/time the response was created (UTC). Example:
            2022-05-27T08:43:12.
        date_modified (Union[Unset, datetime.datetime]): The date/time the response was modified (UTC). Example:
            2022-05-27T08:43:12.
        sections (Union[Unset, List['Section']]): An array of sections from the survey, otherwise `questions`.
        questions (Union[Unset, List['QuestionResponse']]): An array of questions from the survey, otherwise `sections`.
    """

    participant_id: str
    submission_id: str
    field_id: Union[Unset, str] = UNSET
    date_created: Union[Unset, datetime.datetime] = UNSET
    date_modified: Union[Unset, datetime.datetime] = UNSET
    sections: Union[Unset, List["Section"]] = UNSET
    questions: Union[Unset, List["QuestionResponse"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        participant_id = self.participant_id
        submission_id = self.submission_id
        field_id = self.field_id
        date_created: Union[Unset, str] = UNSET
        if not isinstance(self.date_created, Unset):
            date_created = self.date_created.isoformat()

        date_modified: Union[Unset, str] = UNSET
        if not isinstance(self.date_modified, Unset):
            date_modified = self.date_modified.isoformat()

        sections: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.sections, Unset):
            sections = []
            for sections_item_data in self.sections:
                sections_item = sections_item_data.to_dict()

                sections.append(sections_item)

        questions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.questions, Unset):
            questions = []
            for questions_item_data in self.questions:
                questions_item = questions_item_data.to_dict()

                questions.append(questions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "participant_id": participant_id,
                "submission_id": submission_id,
            }
        )
        if field_id is not UNSET:
            field_dict["_id"] = field_id
        if date_created is not UNSET:
            field_dict["date_created"] = date_created
        if date_modified is not UNSET:
            field_dict["date_modified"] = date_modified
        if sections is not UNSET:
            field_dict["sections"] = sections
        if questions is not UNSET:
            field_dict["questions"] = questions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.section import Section
        from ..models.question_response import QuestionResponse

        d = src_dict.copy()
        participant_id = d.pop("participant_id")

        submission_id = d.pop("submission_id")

        field_id = d.pop("_id", UNSET)

        _date_created = d.pop("date_created", UNSET)
        date_created: Union[Unset, datetime.datetime]
        if isinstance(_date_created, Unset):
            date_created = UNSET
        else:
            date_created = isoparse(_date_created)

        _date_modified = d.pop("date_modified", UNSET)
        date_modified: Union[Unset, datetime.datetime]
        if isinstance(_date_modified, Unset):
            date_modified = UNSET
        else:
            date_modified = isoparse(_date_modified)

        sections = []
        _sections = d.pop("sections", UNSET)
        for sections_item_data in _sections or []:
            sections_item = Section.from_dict(sections_item_data)

            sections.append(sections_item)

        questions = []
        _questions = d.pop("questions", UNSET)
        for questions_item_data in _questions or []:
            questions_item = QuestionResponse.from_dict(questions_item_data)

            questions.append(questions_item)

        response_out = cls(
            participant_id=participant_id,
            submission_id=submission_id,
            field_id=field_id,
            date_created=date_created,
            date_modified=date_modified,
            sections=sections,
            questions=questions,
        )

        response_out.additional_properties = d
        return response_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class BaseStudyProlificIdOption(str, Enum):
    NOT_REQUIRED = "not_required"
    QUESTION = "question"
    URL_PARAMETERS = "url_parameters"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.invitation_status import InvitationStatus
from typing import Union
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.invitation_invitee import InvitationInvitee


T = TypeVar("T", bound="Invitation")


@_attrs_define
class Invitation:
    """
    Attributes:
        association (Union[Unset, str]): The ID of the workspace or project to which the invitee was invited.
        invitee (Union[Unset, InvitationInvitee]):
        invited_by (Union[Unset, str]): The ID of the user who sent the invitation.
        status (Union[Unset, InvitationStatus]): The current status of the invitation.
        invite_link (Union[Unset, str]): The link that the invitee can use to accept the invitation.
    """

    association: Union[Unset, str] = UNSET
    invitee: Union[Unset, "InvitationInvitee"] = UNSET
    invited_by: Union[Unset, str] = UNSET
    status: Union[Unset, InvitationStatus] = UNSET
    invite_link: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        association = self.association
        invitee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.invitee, Unset):
            invitee = self.invitee.to_dict()

        invited_by = self.invited_by
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        invite_link = self.invite_link

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if association is not UNSET:
            field_dict["association"] = association
        if invitee is not UNSET:
            field_dict["invitee"] = invitee
        if invited_by is not UNSET:
            field_dict["invited_by"] = invited_by
        if status is not UNSET:
            field_dict["status"] = status
        if invite_link is not UNSET:
            field_dict["invite_link"] = invite_link

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.invitation_invitee import InvitationInvitee

        d = src_dict.copy()
        association = d.pop("association", UNSET)

        _invitee = d.pop("invitee", UNSET)
        invitee: Union[Unset, InvitationInvitee]
        if isinstance(_invitee, Unset):
            invitee = UNSET
        else:
            invitee = InvitationInvitee.from_dict(_invitee)

        invited_by = d.pop("invited_by", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, InvitationStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = InvitationStatus(_status)

        invite_link = d.pop("invite_link", UNSET)

        invitation = cls(
            association=association,
            invitee=invitee,
            invited_by=invited_by,
            status=status,
            invite_link=invite_link,
        )

        invitation.additional_properties = d
        return invitation

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from typing import List
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.participant_group_feeder_studies_item_feeder_completion_codes_item import (
        ParticipantGroupFeederStudiesItemFeederCompletionCodesItem,
    )


T = TypeVar("T", bound="ParticipantGroupFeederStudiesItem")


@_attrs_define
class ParticipantGroupFeederStudiesItem:
    """
    Attributes:
        id (Union[Unset, str]): The id of the study. Example: 5e9b9c9b0f9c9a0001b0b1f4.
        name (Union[Unset, str]): The name of the study. Example: Study 1.
        internal_name (Union[Unset, str]): The internal name of the study. Example: My Study.
        status (Union[Unset, str]): The current status of the study. Example: COMPLETED.
        feeder_completion_codes (Union[Unset, List['ParticipantGroupFeederStudiesItemFeederCompletionCodesItem']]): The
            completion codes which will modify the participants in this group.
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    internal_name: Union[Unset, str] = UNSET
    status: Union[Unset, str] = UNSET
    feeder_completion_codes: Union[
        Unset, List["ParticipantGroupFeederStudiesItemFeederCompletionCodesItem"]
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        internal_name = self.internal_name
        status = self.status
        feeder_completion_codes: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.feeder_completion_codes, Unset):
            feeder_completion_codes = []
            for feeder_completion_codes_item_data in self.feeder_completion_codes:
                feeder_completion_codes_item = (
                    feeder_completion_codes_item_data.to_dict()
                )

                feeder_completion_codes.append(feeder_completion_codes_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if internal_name is not UNSET:
            field_dict["internal_name"] = internal_name
        if status is not UNSET:
            field_dict["status"] = status
        if feeder_completion_codes is not UNSET:
            field_dict["feeder_completion_codes"] = feeder_completion_codes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.participant_group_feeder_studies_item_feeder_completion_codes_item import (
            ParticipantGroupFeederStudiesItemFeederCompletionCodesItem,
        )

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        internal_name = d.pop("internal_name", UNSET)

        status = d.pop("status", UNSET)

        feeder_completion_codes = []
        _feeder_completion_codes = d.pop("feeder_completion_codes", UNSET)
        for feeder_completion_codes_item_data in _feeder_completion_codes or []:
            feeder_completion_codes_item = (
                ParticipantGroupFeederStudiesItemFeederCompletionCodesItem.from_dict(
                    feeder_completion_codes_item_data
                )
            )

            feeder_completion_codes.append(feeder_completion_codes_item)

        participant_group_feeder_studies_item = cls(
            id=id,
            name=name,
            internal_name=internal_name,
            status=status,
            feeder_completion_codes=feeder_completion_codes,
        )

        participant_group_feeder_studies_item.additional_properties = d
        return participant_group_feeder_studies_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="EventType")


@_attrs_define
class EventType:
    """
    Example:
        {'event_type': 'study.status.change', 'description': 'A status change event for studies'}

    Attributes:
        event_type (Union[Unset, str]):
        description (Union[Unset, str]):
    """

    event_type: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        event_type = self.event_type
        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if event_type is not UNSET:
            field_dict["event_type"] = event_type
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        event_type = d.pop("event_type", UNSET)

        description = d.pop("description", UNSET)

        event_type = cls(
            event_type=event_type,
            description=description,
        )

        event_type.additional_properties = d
        return event_type

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...types import UNSET, Unset
from typing import Union
from ...models.requirements_response import RequirementsResponse
from typing import Dict


def _get_kwargs(
    *,
    authorization: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    headers = {}
    if not isinstance(authorization, Unset):
        headers["Authorization"] = authorization

    return {
        "method": "get",
        "url": "/api/v1/eligibility-requirements/",
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[RequirementsResponse]:
    if response.status_code == HTTPStatus.OK:
        response_200 = RequirementsResponse.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[RequirementsResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    authorization: Union[Unset, str] = UNSET,
) -> Response[RequirementsResponse]:
    """Get list of all requirements.

     This endpoint is deprecated - please use [Filters](https://docs.prolific.com/docs/api-
    docs/public/#tag/Filters) instead.

    Get a list of all the requirements defined by Prolific that can
    be used to filter participants.

    Args:
        authorization (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[RequirementsResponse]
    """

    kwargs = _get_kwargs(
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    authorization: Union[Unset, str] = UNSET,
) -> Optional[RequirementsResponse]:
    """Get list of all requirements.

     This endpoint is deprecated - please use [Filters](https://docs.prolific.com/docs/api-
    docs/public/#tag/Filters) instead.

    Get a list of all the requirements defined by Prolific that can
    be used to filter participants.

    Args:
        authorization (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        RequirementsResponse
    """

    return sync_detailed(
        client=client,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    authorization: Union[Unset, str] = UNSET,
) -> Response[RequirementsResponse]:
    """Get list of all requirements.

     This endpoint is deprecated - please use [Filters](https://docs.prolific.com/docs/api-
    docs/public/#tag/Filters) instead.

    Get a list of all the requirements defined by Prolific that can
    be used to filter participants.

    Args:
        authorization (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[RequirementsResponse]
    """

    kwargs = _get_kwargs(
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    authorization: Union[Unset, str] = UNSET,
) -> Optional[RequirementsResponse]:
    """Get list of all requirements.

     This endpoint is deprecated - please use [Filters](https://docs.prolific.com/docs/api-
    docs/public/#tag/Filters) instead.

    Get a list of all the requirements defined by Prolific that can
    be used to filter participants.

    Args:
        authorization (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        RequirementsResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            authorization=authorization,
        )
    ).parsed



from enum import Enum


class SelectFilterListAttributesDataType(str, Enum):
    CHOICEID = "ChoiceID"
    PARTICIPANTGROUPID = "ParticipantGroupID"
    PARTICIPANTID = "ParticipantID"
    STUDYID = "StudyID"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Optional, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast
from typing import Union
from typing import cast, Union
from typing import cast, List
from ..types import UNSET, Unset
from typing import Optional
from typing import Dict

if TYPE_CHECKING:
    from ..models.attribute_value_type_1_item_type_1 import AttributeValueType1ItemType1
    from ..models.attribute_value_type_0 import AttributeValueType0


T = TypeVar("T", bound="Attribute")


@_attrs_define
class Attribute:
    """
    Attributes:
        value (Union['AttributeValueType0', List[Union['AttributeValueType1ItemType1', str]], bool, float, str]):
            Attribute value. It can be null, a number, a date or a boolean depending on the type of requirement.
            For boolean values, there is no need to specify all false (non selected) options. Example: true.
        index (Union[Unset, float]): Attribute index. It is mandatory for some type of requirements,
            like SelectAnswerEligibilityRequirement. Example: 5.0.
        name (Union[Unset, str]): Attribute name. It is mandatory for some type of requirements,
            like AgeRangeEligibilityRequirement.
    """

    value: Union[
        "AttributeValueType0",
        List[Union["AttributeValueType1ItemType1", str]],
        bool,
        float,
        str,
    ]
    index: Union[Unset, float] = UNSET
    name: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.attribute_value_type_1_item_type_1 import (
            AttributeValueType1ItemType1,
        )
        from ..models.attribute_value_type_0 import AttributeValueType0

        value: Union[Dict[str, Any], List[Union[Dict[str, Any], str]], bool, float, str]

        if isinstance(self.value, AttributeValueType0):
            value = self.value.to_dict() if self.value else None

        elif isinstance(self.value, list):
            if self.value is None:
                value = None
            else:
                value = []
                for value_type_1_item_data in self.value:
                    value_type_1_item: Union[Dict[str, Any], str]

                    if isinstance(value_type_1_item_data, AttributeValueType1ItemType1):
                        value_type_1_item = value_type_1_item_data.to_dict()

                    else:
                        value_type_1_item = value_type_1_item_data

                    value.append(value_type_1_item)

        else:
            value = self.value

        index = self.index
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "value": value,
            }
        )
        if index is not UNSET:
            field_dict["index"] = index
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.attribute_value_type_1_item_type_1 import (
            AttributeValueType1ItemType1,
        )
        from ..models.attribute_value_type_0 import AttributeValueType0

        d = src_dict.copy()

        def _parse_value(
            data: object,
        ) -> Union[
            "AttributeValueType0",
            List[Union["AttributeValueType1ItemType1", str]],
            bool,
            float,
            str,
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                _value_type_0 = data
                value_type_0: Optional[AttributeValueType0]
                if _value_type_0 is None:
                    value_type_0 = UNSET
                else:
                    value_type_0 = AttributeValueType0.from_dict(_value_type_0)

                return value_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                value_type_1 = UNSET
                _value_type_1 = data
                for value_type_1_item_data in _value_type_1 or []:

                    def _parse_value_type_1_item(
                        data: object,
                    ) -> Union["AttributeValueType1ItemType1", str]:
                        try:
                            if not isinstance(data, dict):
                                raise TypeError()
                            value_type_1_item_type_1 = (
                                AttributeValueType1ItemType1.from_dict(data)
                            )

                            return value_type_1_item_type_1
                        except:  # noqa: E722
                            pass
                        return cast(Union["AttributeValueType1ItemType1", str], data)

                    value_type_1_item = _parse_value_type_1_item(value_type_1_item_data)

                    value_type_1.append(value_type_1_item)

                return value_type_1
            except:  # noqa: E722
                pass
            return cast(
                Union[
                    "AttributeValueType0",
                    List[Union["AttributeValueType1ItemType1", str]],
                    bool,
                    float,
                    str,
                ],
                data,
            )

        value = _parse_value(d.pop("value"))

        index = d.pop("index", UNSET)

        name = d.pop("name", UNSET)

        attribute = cls(
            value=value,
            index=index,
            name=name,
        )

        attribute.additional_properties = d
        return attribute

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field


from typing import Dict

if TYPE_CHECKING:
    from ..models.range_filter_selected_range import RangeFilterSelectedRange


T = TypeVar("T", bound="RangeFilter")


@_attrs_define
class RangeFilter:
    r"""
    Attributes:
        filter_id (str): ID of the "range" type filter.
        selected_range (RangeFilterSelectedRange): This schema applies for filters of the `range` type, as defined in
            the [filter list response](\#tag/Filters/paths/~1api~1v1~1filters~1/get).

            A dictionary with two possible objects, 'lower' and 'upper'. At least one must be present and a non-null value.

            The expected data type for these values is defined by the `range` filter's `data_type` (see response linked
            above).

            If the data_type is a date, string format should be a parseable ISO8601 date string. Date values should be
            provided as a string in ISO 8601 format.

            Leaving a value as null will result in that bound being set to the lowest or highest possible value, depending
            on whether it is the upper or lower bound.
    """

    filter_id: str
    selected_range: "RangeFilterSelectedRange"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        filter_id = self.filter_id
        selected_range = self.selected_range.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "filter_id": filter_id,
                "selected_range": selected_range,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.range_filter_selected_range import RangeFilterSelectedRange

        d = src_dict.copy()
        filter_id = d.pop("filter_id")

        selected_range = RangeFilterSelectedRange.from_dict(d.pop("selected_range"))

        range_filter = cls(
            filter_id=filter_id,
            selected_range=selected_range,
        )

        range_filter.additional_properties = d
        return range_filter

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from typing import Union
from typing import List
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.range_filter import RangeFilter
    from ..models.select_filter import SelectFilter


T = TypeVar("T", bound="CloneFilterSetResponse201")


@_attrs_define
class CloneFilterSetResponse201:
    """
    Attributes:
        id (Union[Unset, str]): ID of the filter set.
        version (Union[Unset, int]): An incrementing integer indicating the version of the filter set.
        is_deleted (Union[Unset, bool]): Whether the filter set has been deleted.
        is_locked (Union[Unset, bool]): Whether the filter set has been locked.
        workspace_id (Union[Unset, None, str]): ID of the workspace where the filter set can be used.
        organisation_id (Union[Unset, None, str]): ID of the workspace where the filter set can be used.
        name (Union[Unset, str]): Name of the filter set.
        filters (Union[Unset, List[Union['RangeFilter', 'SelectFilter']]]): List of all filters contained in the filter
            set.
        eligible_participant_count (Union[Unset, int]): The number of participants who match the filter sets filters.
            Please note that if the number is
            lower than 25 the count will be obscured to prevent identification of participants.
    """

    id: Union[Unset, str] = UNSET
    version: Union[Unset, int] = UNSET
    is_deleted: Union[Unset, bool] = UNSET
    is_locked: Union[Unset, bool] = UNSET
    workspace_id: Union[Unset, None, str] = UNSET
    organisation_id: Union[Unset, None, str] = UNSET
    name: Union[Unset, str] = UNSET
    filters: Union[Unset, List[Union["RangeFilter", "SelectFilter"]]] = UNSET
    eligible_participant_count: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.select_filter import SelectFilter

        id = self.id
        version = self.version
        is_deleted = self.is_deleted
        is_locked = self.is_locked
        workspace_id = self.workspace_id
        organisation_id = self.organisation_id
        name = self.name
        filters: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = []
            for filters_item_data in self.filters:
                filters_item: Dict[str, Any]

                if isinstance(filters_item_data, SelectFilter):
                    filters_item = filters_item_data.to_dict()

                else:
                    filters_item = filters_item_data.to_dict()

                filters.append(filters_item)

        eligible_participant_count = self.eligible_participant_count

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if version is not UNSET:
            field_dict["version"] = version
        if is_deleted is not UNSET:
            field_dict["is_deleted"] = is_deleted
        if is_locked is not UNSET:
            field_dict["is_locked"] = is_locked
        if workspace_id is not UNSET:
            field_dict["workspace_id"] = workspace_id
        if organisation_id is not UNSET:
            field_dict["organisation_id"] = organisation_id
        if name is not UNSET:
            field_dict["name"] = name
        if filters is not UNSET:
            field_dict["filters"] = filters
        if eligible_participant_count is not UNSET:
            field_dict["eligible_participant_count"] = eligible_participant_count

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.range_filter import RangeFilter
        from ..models.select_filter import SelectFilter

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        version = d.pop("version", UNSET)

        is_deleted = d.pop("is_deleted", UNSET)

        is_locked = d.pop("is_locked", UNSET)

        workspace_id = d.pop("workspace_id", UNSET)

        organisation_id = d.pop("organisation_id", UNSET)

        name = d.pop("name", UNSET)

        filters = []
        _filters = d.pop("filters", UNSET)
        for filters_item_data in _filters or []:

            def _parse_filters_item(
                data: object,
            ) -> Union["RangeFilter", "SelectFilter"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    filters_item_type_0 = SelectFilter.from_dict(data)

                    return filters_item_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                filters_item_type_1 = RangeFilter.from_dict(data)

                return filters_item_type_1

            filters_item = _parse_filters_item(filters_item_data)

            filters.append(filters_item)

        eligible_participant_count = d.pop("eligible_participant_count", UNSET)

        clone_filter_set_response_201 = cls(
            id=id,
            version=version,
            is_deleted=is_deleted,
            is_locked=is_locked,
            workspace_id=workspace_id,
            organisation_id=organisation_id,
            name=name,
            filters=filters,
            eligible_participant_count=eligible_participant_count,
        )

        clone_filter_set_response_201.additional_properties = d
        return clone_filter_set_response_201

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="BulkBonus")


@_attrs_define
class BulkBonus:
    """
    Example:
        {'id': '621014cb8e9e0f81e387021f', 'study': '620ca2735fcbba4fa2b3211a', 'amount': 850, 'fees': 283.34, 'vat': 0,
            'total_amount': 1133.34}

    Attributes:
        id (str): Bonus ID. It is the ID to be used when paying the bonus
        total_amount (float): Total ammount that will be deducted from your balance in cents
        study (Union[Unset, str]): Study ID
        amount (Union[Unset, float]): The amount the participant will receive in cents
        fees (Union[Unset, float]): The fees Prolific will charge for this bonus in cents
        vat (Union[Unset, float]): The VAT cost for this bonus in cents
    """

    id: str
    total_amount: float
    study: Union[Unset, str] = UNSET
    amount: Union[Unset, float] = UNSET
    fees: Union[Unset, float] = UNSET
    vat: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        total_amount = self.total_amount
        study = self.study
        amount = self.amount
        fees = self.fees
        vat = self.vat

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "total_amount": total_amount,
            }
        )
        if study is not UNSET:
            field_dict["study"] = study
        if amount is not UNSET:
            field_dict["amount"] = amount
        if fees is not UNSET:
            field_dict["fees"] = fees
        if vat is not UNSET:
            field_dict["vat"] = vat

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        total_amount = d.pop("total_amount")

        study = d.pop("study", UNSET)

        amount = d.pop("amount", UNSET)

        fees = d.pop("fees", UNSET)

        vat = d.pop("vat", UNSET)

        bulk_bonus = cls(
            id=id,
            total_amount=total_amount,
            study=study,
            amount=amount,
            fees=fees,
            vat=vat,
        )

        bulk_bonus.additional_properties = d
        return bulk_bonus

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from typing import Dict
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.workspace_balance_available_balance_breakdown import (
        WorkspaceBalanceAvailableBalanceBreakdown,
    )
    from ..models.workspace_balance_balance_breakdown import (
        WorkspaceBalanceBalanceBreakdown,
    )


T = TypeVar("T", bound="WorkspaceBalance")


@_attrs_define
class WorkspaceBalance:
    """
    Example:
        {'currency_code': 'GBP', 'total_balance': 630, 'balance_breakdown': {'rewards': 450, 'fees': 150, 'vat': 30},
            'available_balance': 210, 'available_balance_breakdown': {'rewards': 150, 'fees': 50, 'vat': 10}}

    Attributes:
        currency_code (Union[Unset, str]): The currency used for all financial transactions within the workspace.
        total_balance (Union[Unset, int]): The total balance of the workspace, including funds which have already been
            assigned to active studies.

            All monetary values are shown in the sub-currency of your workspace currency (e.g. pence, cents).
        balance_breakdown (Union[Unset, WorkspaceBalanceBalanceBreakdown]): A breakdown of the total balance of the
            workspace into:
            - Funds available to pay to participants
            - Funds pre-paid to Prolific for our services
            - Funds for any VAT applied to our service fees
        available_balance (Union[Unset, int]): The remaining balance of your workspace which is available to spend,
            after removing funds assigned to already active studies, etc.
        available_balance_breakdown (Union[Unset, WorkspaceBalanceAvailableBalanceBreakdown]): A breakdown of the
            available balance of the workspace into:
            - Funds available to pay to participants
            - Funds pre-paid to Prolific for our services
            - Funds for any VAT applied to our service fees
    """

    currency_code: Union[Unset, str] = UNSET
    total_balance: Union[Unset, int] = UNSET
    balance_breakdown: Union[Unset, "WorkspaceBalanceBalanceBreakdown"] = UNSET
    available_balance: Union[Unset, int] = UNSET
    available_balance_breakdown: Union[
        Unset, "WorkspaceBalanceAvailableBalanceBreakdown"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        currency_code = self.currency_code
        total_balance = self.total_balance
        balance_breakdown: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.balance_breakdown, Unset):
            balance_breakdown = self.balance_breakdown.to_dict()

        available_balance = self.available_balance
        available_balance_breakdown: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.available_balance_breakdown, Unset):
            available_balance_breakdown = self.available_balance_breakdown.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if currency_code is not UNSET:
            field_dict["currency_code"] = currency_code
        if total_balance is not UNSET:
            field_dict["total_balance"] = total_balance
        if balance_breakdown is not UNSET:
            field_dict["balance_breakdown"] = balance_breakdown
        if available_balance is not UNSET:
            field_dict["available_balance"] = available_balance
        if available_balance_breakdown is not UNSET:
            field_dict["available_balance_breakdown"] = available_balance_breakdown

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.workspace_balance_available_balance_breakdown import (
            WorkspaceBalanceAvailableBalanceBreakdown,
        )
        from ..models.workspace_balance_balance_breakdown import (
            WorkspaceBalanceBalanceBreakdown,
        )

        d = src_dict.copy()
        currency_code = d.pop("currency_code", UNSET)

        total_balance = d.pop("total_balance", UNSET)

        _balance_breakdown = d.pop("balance_breakdown", UNSET)
        balance_breakdown: Union[Unset, WorkspaceBalanceBalanceBreakdown]
        if isinstance(_balance_breakdown, Unset):
            balance_breakdown = UNSET
        else:
            balance_breakdown = WorkspaceBalanceBalanceBreakdown.from_dict(
                _balance_breakdown
            )

        available_balance = d.pop("available_balance", UNSET)

        _available_balance_breakdown = d.pop("available_balance_breakdown", UNSET)
        available_balance_breakdown: Union[
            Unset, WorkspaceBalanceAvailableBalanceBreakdown
        ]
        if isinstance(_available_balance_breakdown, Unset):
            available_balance_breakdown = UNSET
        else:
            available_balance_breakdown = (
                WorkspaceBalanceAvailableBalanceBreakdown.from_dict(
                    _available_balance_breakdown
                )
            )

        workspace_balance = cls(
            currency_code=currency_code,
            total_balance=total_balance,
            balance_breakdown=balance_breakdown,
            available_balance=available_balance,
            available_balance_breakdown=available_balance_breakdown,
        )

        workspace_balance.additional_properties = d
        return workspace_balance

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, cast

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.study_short_study_type import StudyShortStudyType
from typing import Union
import datetime
from ..types import UNSET, Unset
from ..models.study_short_status import StudyShortStatus
from dateutil.parser import isoparse


T = TypeVar("T", bound="StudyShort")


@_attrs_define
class StudyShort:
    """
    Example:
        {'id': '60d9aadeb86739de712faee0', 'name': "Study about API's", 'status': 'UNPUBLISHED'}

    Attributes:
        id (str): Study id. It is created by Prolific.
        name (str): Public name or title of the study
        internal_name (Union[Unset, None, str]): Internal name of the study, not shown to participants
        status (Union[Unset, StudyShortStatus]): Status of the study.
        study_type (Union[Unset, StudyShortStudyType]): Deprecated. Type of study.
        total_available_places (Union[Unset, float]): How many participants are you looking to recruit
        places_taken (Union[Unset, float]): Places already taken, number of submission started excluding timed out and
            returned submissions
        number_of_submissions (Union[Unset, float]):
        reward (Union[Unset, float]): How much are you going to pay the participants in cents. We use the currency of
            your account
        total_cost (Union[Unset, float]): Total cost of the study including fees
        published_at (Union[Unset, None, datetime.datetime]): Date time when the study was published.
        publish_at (Union[Unset, None, datetime.datetime]): Date time when the study was scheduled to be published.
        date_created (Union[Unset, datetime.datetime]): Date time when the study was created
    """

    id: str
    name: str
    internal_name: Union[Unset, None, str] = UNSET
    status: Union[Unset, StudyShortStatus] = UNSET
    study_type: Union[Unset, StudyShortStudyType] = UNSET
    total_available_places: Union[Unset, float] = UNSET
    places_taken: Union[Unset, float] = UNSET
    number_of_submissions: Union[Unset, float] = UNSET
    reward: Union[Unset, float] = UNSET
    total_cost: Union[Unset, float] = UNSET
    published_at: Union[Unset, None, datetime.datetime] = UNSET
    publish_at: Union[Unset, None, datetime.datetime] = UNSET
    date_created: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        internal_name = self.internal_name
        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        study_type: Union[Unset, str] = UNSET
        if not isinstance(self.study_type, Unset):
            study_type = self.study_type.value

        total_available_places = self.total_available_places
        places_taken = self.places_taken
        number_of_submissions = self.number_of_submissions
        reward = self.reward
        total_cost = self.total_cost
        published_at: Union[Unset, None, str] = UNSET
        if not isinstance(self.published_at, Unset):
            published_at = self.published_at.isoformat() if self.published_at else None

        publish_at: Union[Unset, None, str] = UNSET
        if not isinstance(self.publish_at, Unset):
            publish_at = self.publish_at.isoformat() if self.publish_at else None

        date_created: Union[Unset, str] = UNSET
        if not isinstance(self.date_created, Unset):
            date_created = self.date_created.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "name": name,
            }
        )
        if internal_name is not UNSET:
            field_dict["internal_name"] = internal_name
        if status is not UNSET:
            field_dict["status"] = status
        if study_type is not UNSET:
            field_dict["study_type"] = study_type
        if total_available_places is not UNSET:
            field_dict["total_available_places"] = total_available_places
        if places_taken is not UNSET:
            field_dict["places_taken"] = places_taken
        if number_of_submissions is not UNSET:
            field_dict["number_of_submissions"] = number_of_submissions
        if reward is not UNSET:
            field_dict["reward"] = reward
        if total_cost is not UNSET:
            field_dict["total_cost"] = total_cost
        if published_at is not UNSET:
            field_dict["published_at"] = published_at
        if publish_at is not UNSET:
            field_dict["publish_at"] = publish_at
        if date_created is not UNSET:
            field_dict["date_created"] = date_created

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        internal_name = d.pop("internal_name", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, StudyShortStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = StudyShortStatus(_status)

        _study_type = d.pop("study_type", UNSET)
        study_type: Union[Unset, StudyShortStudyType]
        if isinstance(_study_type, Unset):
            study_type = UNSET
        else:
            study_type = StudyShortStudyType(_study_type)

        total_available_places = d.pop("total_available_places", UNSET)

        places_taken = d.pop("places_taken", UNSET)

        number_of_submissions = d.pop("number_of_submissions", UNSET)

        reward = d.pop("reward", UNSET)

        total_cost = d.pop("total_cost", UNSET)

        _published_at = d.pop("published_at", UNSET)
        published_at: Union[Unset, None, datetime.datetime]
        if _published_at is None:
            published_at = None
        elif isinstance(_published_at, Unset):
            published_at = UNSET
        else:
            try:
                published_at = isoparse(_published_at)
            except:
                published_at = cast(Union[None, Unset, datetime.datetime], _published_at)

        _publish_at = d.pop("publish_at", UNSET)
        publish_at: Union[Unset, None, datetime.datetime]
        if _publish_at is None:
            publish_at = None
        elif isinstance(_publish_at, Unset):
            publish_at = UNSET
        else:
            try:
                publish_at = isoparse(_publish_at)
            except:
                publish_at = cast(Union[None, Unset, datetime.datetime], _publish_at)

        _date_created = d.pop("date_created", UNSET)
        date_created: Union[Unset, datetime.datetime]
        if isinstance(_date_created, Unset):
            date_created = UNSET
        else:
            try:
                date_created = isoparse(_date_created)
            except:
                date_created = cast(Union[None, Unset, datetime.datetime], _date_created)

        study_short = cls(
            id=id,
            name=name,
            internal_name=internal_name,
            status=status,
            study_type=study_type,
            total_available_places=total_available_places,
            places_taken=places_taken,
            number_of_submissions=number_of_submissions,
            reward=reward,
            total_cost=total_cost,
            published_at=published_at,
            publish_at=publish_at,
            date_created=date_created,
        )

        study_short.additional_properties = d
        return study_short

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response
from ... import errors

from ...models.submission import Submission
from ...models.submission_transition import SubmissionTransition
from typing import Dict


def _get_kwargs(
    id: str,
    *,
    json_body: SubmissionTransition,
    authorization: str,
) -> Dict[str, Any]:
    headers = {}
    headers["Authorization"] = authorization

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": "/api/v1/submissions/{id}/transition/".format(
            id=id,
        ),
        "json": json_json_body,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Submission]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Submission.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Submission]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: SubmissionTransition,
    authorization: str,
) -> Response[Submission]:
    """Approve or reject a submission

     Transition a submission to `APPROVED` or `REJECTED`. Once the status is changed, it can not be
    restored to its previous value.

    We __strongly__ recommend that, when giving approval to a submission through the API, you first
    observe the `submission.status.change`
    [event](https://docs.prolific.com/docs/api-docs/public/#tag/Hooks/paths/~1api~1v1~1hooks~1event-
    types~1/get) for a status transition to
    `AWAITING_REVIEW` before making the approval request. Our system is currently unable to process
    approvals before this transition.
    Note this endpoint is idempotent, so if you make the same request twice, the second request will be
    ignored.

    Args:
        id (str):
        authorization (str):
        json_body (SubmissionTransition):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Submission]
    """

    kwargs = _get_kwargs(
        id=id,
        json_body=json_body,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: SubmissionTransition,
    authorization: str,
) -> Optional[Submission]:
    """Approve or reject a submission

     Transition a submission to `APPROVED` or `REJECTED`. Once the status is changed, it can not be
    restored to its previous value.

    We __strongly__ recommend that, when giving approval to a submission through the API, you first
    observe the `submission.status.change`
    [event](https://docs.prolific.com/docs/api-docs/public/#tag/Hooks/paths/~1api~1v1~1hooks~1event-
    types~1/get) for a status transition to
    `AWAITING_REVIEW` before making the approval request. Our system is currently unable to process
    approvals before this transition.
    Note this endpoint is idempotent, so if you make the same request twice, the second request will be
    ignored.

    Args:
        id (str):
        authorization (str):
        json_body (SubmissionTransition):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Submission
    """

    return sync_detailed(
        id=id,
        client=client,
        json_body=json_body,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: SubmissionTransition,
    authorization: str,
) -> Response[Submission]:
    """Approve or reject a submission

     Transition a submission to `APPROVED` or `REJECTED`. Once the status is changed, it can not be
    restored to its previous value.

    We __strongly__ recommend that, when giving approval to a submission through the API, you first
    observe the `submission.status.change`
    [event](https://docs.prolific.com/docs/api-docs/public/#tag/Hooks/paths/~1api~1v1~1hooks~1event-
    types~1/get) for a status transition to
    `AWAITING_REVIEW` before making the approval request. Our system is currently unable to process
    approvals before this transition.
    Note this endpoint is idempotent, so if you make the same request twice, the second request will be
    ignored.

    Args:
        id (str):
        authorization (str):
        json_body (SubmissionTransition):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Submission]
    """

    kwargs = _get_kwargs(
        id=id,
        json_body=json_body,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: str,
    *,
    client: AuthenticatedClient,
    json_body: SubmissionTransition,
    authorization: str,
) -> Optional[Submission]:
    """Approve or reject a submission

     Transition a submission to `APPROVED` or `REJECTED`. Once the status is changed, it can not be
    restored to its previous value.

    We __strongly__ recommend that, when giving approval to a submission through the API, you first
    observe the `submission.status.change`
    [event](https://docs.prolific.com/docs/api-docs/public/#tag/Hooks/paths/~1api~1v1~1hooks~1event-
    types~1/get) for a status transition to
    `AWAITING_REVIEW` before making the approval request. Our system is currently unable to process
    approvals before this transition.
    Note this endpoint is idempotent, so if you make the same request twice, the second request will be
    ignored.

    Args:
        id (str):
        authorization (str):
        json_body (SubmissionTransition):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Submission
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            json_body=json_body,
            authorization=authorization,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="DuplicateStudyJsonBody")


@_attrs_define
class DuplicateStudyJsonBody:
    """
    Attributes:
        block_previous_participants (Union[Unset, bool]): Controls whether the block list is added or not. Default:
            True.
    """

    block_previous_participants: Union[Unset, bool] = True
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        block_previous_participants = self.block_previous_participants

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if block_previous_participants is not UNSET:
            field_dict["block_previous_participants"] = block_previous_participants

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        block_previous_participants = d.pop("block_previous_participants", UNSET)

        duplicate_study_json_body = cls(
            block_previous_participants=block_previous_participants,
        )

        duplicate_study_json_body.additional_properties = d
        return duplicate_study_json_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.filter_list_attributes_type import FilterListAttributesType
from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="FilterListAttributes")


@_attrs_define
class FilterListAttributes:
    """
    Attributes:
        filter_id (Union[Unset, str]): The ID of the filter, based on a slugified version of the title at the time the
            filter was created.
        title (Union[Unset, str]): The title of the filter.
        description (Union[Unset, str]): A description of the filter.
        type (Union[Unset, FilterListAttributesType]): The filter type.
        question (Union[Unset, str]): The question asked of participants to generate this filter.
    """

    filter_id: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    type: Union[Unset, FilterListAttributesType] = UNSET
    question: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        filter_id = self.filter_id
        title = self.title
        description = self.description
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        question = self.question

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if filter_id is not UNSET:
            field_dict["filter_id"] = filter_id
        if title is not UNSET:
            field_dict["title"] = title
        if description is not UNSET:
            field_dict["description"] = description
        if type is not UNSET:
            field_dict["type"] = type
        if question is not UNSET:
            field_dict["question"] = question

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        filter_id = d.pop("filter_id", UNSET)

        title = d.pop("title", UNSET)

        description = d.pop("description", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, FilterListAttributesType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = FilterListAttributesType(_type)

        question = d.pop("question", UNSET)

        filter_list_attributes = cls(
            filter_id=filter_id,
            title=title,
            description=description,
            type=type,
            question=question,
        )

        filter_list_attributes.additional_properties = d
        return filter_list_attributes

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from typing import Union
from typing import List
from ..types import UNSET, Unset
from typing import Dict

if TYPE_CHECKING:
    from ..models.range_filter import RangeFilter
    from ..models.select_filter import SelectFilter


T = TypeVar("T", bound="UpdateFilterSet")


@_attrs_define
class UpdateFilterSet:
    """
    Example:
        [{'name': 'Left-handed 30-somethings', 'filters': [{'id': 'handedness', 'selected_values': ['1']}, {'id': 'age',
            'selected_range': {'lower': 30, 'upper': 39}}]}]

    Attributes:
        name (Union[Unset, str]): Name of the filter set.
        filters (Union[Unset, List[Union['RangeFilter', 'SelectFilter']]]): List of all filters contained in the filter
            set.
    """

    name: Union[Unset, str] = UNSET
    filters: Union[Unset, List[Union["RangeFilter", "SelectFilter"]]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.select_filter import SelectFilter

        name = self.name
        filters: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = []
            for filters_item_data in self.filters:
                filters_item: Dict[str, Any]

                if isinstance(filters_item_data, SelectFilter):
                    filters_item = filters_item_data.to_dict()

                else:
                    filters_item = filters_item_data.to_dict()

                filters.append(filters_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if filters is not UNSET:
            field_dict["filters"] = filters

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.range_filter import RangeFilter
        from ..models.select_filter import SelectFilter

        d = src_dict.copy()
        name = d.pop("name", UNSET)

        filters = []
        _filters = d.pop("filters", UNSET)
        for filters_item_data in _filters or []:

            def _parse_filters_item(
                data: object,
            ) -> Union["RangeFilter", "SelectFilter"]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    filters_item_type_0 = SelectFilter.from_dict(data)

                    return filters_item_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                filters_item_type_1 = RangeFilter.from_dict(data)

                return filters_item_type_1

            filters_item = _parse_filters_item(filters_item_data)

            filters.append(filters_item)

        update_filter_set = cls(
            name=name,
            filters=filters,
        )

        update_filter_set.additional_properties = d
        return update_filter_set

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field


from typing import cast, List


T = TypeVar("T", bound="SendBulkMessage")


@_attrs_define
class SendBulkMessage:
    """
    Attributes:
        ids (List[str]): A list of participant ID's
        body (str): Message Body. Text is sanitised for safe storage and display.
        study_id (str): A study ID
    """

    ids: List[str]
    body: str
    study_id: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        ids = self.ids

        body = self.body
        study_id = self.study_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "ids": ids,
                "body": body,
                "study_id": study_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        ids = cast(List[str], d.pop("ids"))

        body = d.pop("body")

        study_id = d.pop("study_id")

        send_bulk_message = cls(
            ids=ids,
            body=body,
            study_id=study_id,
        )

        send_bulk_message.additional_properties = d
        return send_bulk_message

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="RequirementQuery")


@_attrs_define
class RequirementQuery:
    """
    Attributes:
        id (Union[Unset, str]): Id of the question Example: 54ac6ea9fdf99b2204feb899.
        question (Union[Unset, None, str]): Question asked to the participant Example: What is your first language?.
    """

    id: Union[Unset, str] = UNSET
    question: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        question = self.question

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if question is not UNSET:
            field_dict["question"] = question

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        question = d.pop("question", UNSET)

        requirement_query = cls(
            id=id,
            question=question,
        )

        requirement_query.additional_properties = d
        return requirement_query

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast, List
from typing import Union
from ..types import UNSET, Unset


T = TypeVar("T", bound="FilterListDetailedAttributes")


@_attrs_define
class FilterListDetailedAttributes:
    """
    Attributes:
        researcher_help_text (Union[Unset, None, str]): Some help text to be displayed to researchers in the
            prescreening modal.
        participant_help_text (Union[Unset, None, str]): Some help text to be displayed to participants in the About You
            section.
        category (Union[Unset, None, str]): The category the filter is displayed in in About You and the prescreening
            modal.
        subcategory (Union[Unset, None, str]): The sub-category the filter is displayed in in the prescreening modal.
        display_order (Union[Unset, None, int]): The order in which the filter is displayed within its sub-category in
            the prescreening modal.
        tags (Union[Unset, None, List[str]]): Some additional tags that can be used to display the filter in a specific
            way, e.g. recommended, new, expiring.
    """

    researcher_help_text: Union[Unset, None, str] = UNSET
    participant_help_text: Union[Unset, None, str] = UNSET
    category: Union[Unset, None, str] = UNSET
    subcategory: Union[Unset, None, str] = UNSET
    display_order: Union[Unset, None, int] = UNSET
    tags: Union[Unset, None, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        researcher_help_text = self.researcher_help_text
        participant_help_text = self.participant_help_text
        category = self.category
        subcategory = self.subcategory
        display_order = self.display_order
        tags: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            if self.tags is None:
                tags = None
            else:
                tags = self.tags

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if researcher_help_text is not UNSET:
            field_dict["researcher_help_text"] = researcher_help_text
        if participant_help_text is not UNSET:
            field_dict["participant_help_text"] = participant_help_text
        if category is not UNSET:
            field_dict["category"] = category
        if subcategory is not UNSET:
            field_dict["subcategory"] = subcategory
        if display_order is not UNSET:
            field_dict["display_order"] = display_order
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        researcher_help_text = d.pop("researcher_help_text", UNSET)

        participant_help_text = d.pop("participant_help_text", UNSET)

        category = d.pop("category", UNSET)

        subcategory = d.pop("subcategory", UNSET)

        display_order = d.pop("display_order", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        filter_list_detailed_attributes = cls(
            researcher_help_text=researcher_help_text,
            participant_help_text=participant_help_text,
            category=category,
            subcategory=subcategory,
            display_order=display_order,
            tags=tags,
        )

        filter_list_detailed_attributes.additional_properties = d
        return filter_list_detailed_attributes

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...types import UNSET, Unset
from typing import Union
from ...models.requirements_count import RequirementsCount
from ...models.requirements_count_request import RequirementsCountRequest
from typing import Dict


def _get_kwargs(
    *,
    json_body: RequirementsCountRequest,
    authorization: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    headers = {}
    if not isinstance(authorization, Unset):
        headers["Authorization"] = authorization

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": "/api/v1/eligibility-count/",
        "json": json_json_body,
        "headers": headers,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[RequirementsCount]:
    if response.status_code == HTTPStatus.OK:
        response_200 = RequirementsCount.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[RequirementsCount]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    json_body: RequirementsCountRequest,
    authorization: Union[Unset, str] = UNSET,
) -> Response[RequirementsCount]:
    """Count participants

     Count how many participants meet all the given filters.
    Only participants that pass **every one** of the filters are counted.
    Zero means that there are less than 25 participants. We do not show lower numbers to protect the
    privacy of the participants.

    To see a list of filters that may be passed to this endpoint, see the documentation for filters.

    Args:
        authorization (Union[Unset, str]):
        json_body (RequirementsCountRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[RequirementsCount]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        authorization=authorization,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    json_body: RequirementsCountRequest,
    authorization: Union[Unset, str] = UNSET,
) -> Optional[RequirementsCount]:
    """Count participants

     Count how many participants meet all the given filters.
    Only participants that pass **every one** of the filters are counted.
    Zero means that there are less than 25 participants. We do not show lower numbers to protect the
    privacy of the participants.

    To see a list of filters that may be passed to this endpoint, see the documentation for filters.

    Args:
        authorization (Union[Unset, str]):
        json_body (RequirementsCountRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        RequirementsCount
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
        authorization=authorization,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    json_body: RequirementsCountRequest,
    authorization: Union[Unset, str] = UNSET,
) -> Response[RequirementsCount]:
    """Count participants

     Count how many participants meet all the given filters.
    Only participants that pass **every one** of the filters are counted.
    Zero means that there are less than 25 participants. We do not show lower numbers to protect the
    privacy of the participants.

    To see a list of filters that may be passed to this endpoint, see the documentation for filters.

    Args:
        authorization (Union[Unset, str]):
        json_body (RequirementsCountRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[RequirementsCount]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
        authorization=authorization,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    json_body: RequirementsCountRequest,
    authorization: Union[Unset, str] = UNSET,
) -> Optional[RequirementsCount]:
    """Count participants

     Count how many participants meet all the given filters.
    Only participants that pass **every one** of the filters are counted.
    Zero means that there are less than 25 participants. We do not show lower numbers to protect the
    privacy of the participants.

    To see a list of filters that may be passed to this endpoint, see the documentation for filters.

    Args:
        authorization (Union[Unset, str]):
        json_body (RequirementsCountRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        RequirementsCount
    """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
            authorization=authorization,
        )
    ).parsed
